/* $Id: popen.c,v 3.1 1994/07/26 22:40:41 tom Exp $
 *
 * Imitate the popen() and pclose() functions.
 */
#ifdef __WATCOMC__
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <process.h>
#include <io.h>
#include "souper.h"

static char *command;			/* data piped to this command */
static char pipe_name[FILENAME_MAX];	/* name of the temporary file */

/* Open a pipe for writing.
 */
FILE *
popen (const char *cmd, const char *type)
{
    char *tmpdir;

    /* Save the command. */
    command = xstrdup(cmd);

    /* Set temporary file name. */
    if ((tmpdir = getenv("TMP")) == NULL) {
	pipe_name[0] = '\0';
    } else {
	strcpy(pipe_name, tmpdir);
	strcat(pipe_name, "/");
    }
    strcat(pipe_name, tmpnam(NULL));
 
    /* Open the intermediate file and return the stream. */
    return fopen(pipe_name, type) ;
}
 
/* Close the pipe.
 */
int
pclose (FILE *f)
{
    char *argv[30], **arg, *cmdline, *s, opt[FILENAME_MAX];
    int ostdin, status;
 
    fclose(f);

    /* Split the command into an argument array. */
    arg = argv;
    s = strtok(command, " ");
    *arg++ = s;
    while ((s = strtok(NULL, " ")) != NULL) {
	*arg++ = s;
    }
    *arg = NULL;
 
    /* Redirect the program's stdin. */
    ostdin = dup(fileno(stdin));
    freopen(pipe_name, "r", stdin);
 
    /* Run the program. */
    status = spawnvp(P_WAIT, argv[0], argv);
 
    /* Restore stdin. */
    dup2(ostdin, fileno(stdin));

    free(command);
    unlink(pipe_name);
    return status;
}
#endif /*__WATCOMC__*/
