/* $Id: popen.c,v 3.1 1994/07/26 22:40:41 tom Exp $
 *
 * Imitate the popen() and pclose() functions.
 */
#ifdef __WATCOMC__
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <process.h>
#include <io.h>
#include "souper.h"

static char pipe_name[FILENAME_MAX];	/* name of the temporary file */

/* Open a pipe for reading.
 */
FILE *
popen (const char *cmd, const char *type)
{
    char *tmpdir, *argv[30], **arg, *cmdline, *s, opt[FILENAME_MAX];
    int ostdout, status;

    /* Set temporary file name. */
    if ((tmpdir = getenv("TMP")) == NULL) {
	pipe_name[0] = '\0';
    } else {
	strcpy(pipe_name, tmpdir);
	strcat(pipe_name, "/");
    }
    strcat(pipe_name, tmpnam(NULL));

    /* Split the command into an argument array. */
    cmdline = xstrdup(cmd);
    arg = argv;
    s = strtok(cmdline, " ");
    *arg++ = s;
    while ((s = strtok(NULL, " ")) != NULL) {
	*arg++ = s;
    }
    *arg = NULL;
 
    /* Redirect the program's stdout. */
    ostdout = dup(fileno(stdout));
    freopen(pipe_name, "w", stdout);
 
    /* Run the program. */
    status = spawnvp(P_WAIT, argv[0], argv);
 
    /* Restore stdout. */
    dup2(ostdout, fileno(stdout));
    free(cmdline);

    if (status != 0)
	return NULL;
 
    /* Open the intermediate file and return the stream. */
    return fopen(pipe_name, type) ;
}
 
/* Close the pipe.
 */
int
pclose (FILE *f)
{
    int status;
 
    status = fclose(f);
    unlink(pipe_name);
    return status;
}
#endif /*__WATCOMC__*/
